import java.util.*;

class ServerData {
    
    HashMap<Integer, ArrayList<Position> > positions;

    public ServerData() {
	positions = new HashMap<Integer, ArrayList<Position> >();
    }

    public synchronized boolean exists(int id) {
	if (positions.containsKey(id)) {
	    return true;
	}
	return false;
    }

    public synchronized void newClient(int id) {
	ArrayList<Position> pos = new ArrayList<>();
	positions.put(id,pos);
    }

    public synchronized void removeClient(int id) {
	positions.remove(id);
    }

    public synchronized void addPosition(int id, double x, double y, double z) {
	ArrayList<Position> pos = positions.get(id);
	pos.add(new Position(x,y,z));	
    }

    public synchronized double getPathLength(int id) {
	double length = 0.0;
	boolean first = true;
	Position pOld = null;

	ArrayList<Position> pos = positions.get(id);
	
	for(Position p : pos) {
	    if (first) {
		pOld = p;
		first = false;
	    }
	    else {
		length += pOld.distance(p);
		pOld = p;
	    }
	}
	return length;
    }

    public synchronized boolean findPosition(int id, double x, double y, double z, double prec) {
	ArrayList<Position> pos = positions.get(id);	
	Position loc = new Position(x,y,z);
	boolean ok = false;
	for (Position p : pos) {
	    if (loc.equals(p,prec)) {
		ok = true;
		break;
	    }
	}
	return ok;
    }

    public synchronized double[] getSlope(int id) {
	double z=0,zz=0;
	double[] slope = new double[2];
	slope[0] = 0.0;
	slope[1] = 0.0;
	boolean first = true;
	ArrayList<Position> pos = positions.get(id);
	
	for(Position p : pos) {
	    if (first) {
		z = p.getZ();
		first = false;
	    }
	    else {
		zz = p.getZ();
		if (zz>=z) {
		    slope[0] += zz-z;
		}
		else {
		    slope[1] += z-zz;
		}
		z = zz;
	    }
	}
	return slope;
    }

    public synchronized String getPosOthers(int id) {

	if (positions.size() == 1) {
	    return "no other clients =  no other positions";
	}

	String posOthers = "";
	Set<Integer> ids = positions.keySet();
	for(int i : ids) {
	    if (i != id) {
		ArrayList<Position> pos = positions.get(i);
		Position p = pos.get(pos.size()-1);
		posOthers += p.getX()+","+p.getY()+","+p.getZ()+",";
	    }
	}
	// remove last comma
	posOthers = posOthers.substring(0,posOthers.length()-1);
	return posOthers;
    }
}
