import java.io.*;
import java.net.*;
import java.util.*;

class ThreadServer extends Thread {

    BufferedReader br;
    PrintStream ps;
    Socket sock;
    ServerData data;
    int idClient;

    public ThreadServer(int idClient, Socket sock, ServerData data) {
	this.sock = sock;
	this.idClient = idClient;
	this.data = data;
    }
  
    public void run() {

	try {
	    br = new BufferedReader(new InputStreamReader(sock.getInputStream()));
	    ps = new PrintStream(sock.getOutputStream());
	}
	catch(IOException e) {
	    System.err.println("Thread "+idClient+": cannot create streams. Aborting.");
	    return;
	}
	// create list for idClient
	data.newClient(idClient);
	requestLoop();
	data.removeClient(idClient);
	System.out.println("end of thread "+idClient);
    }

    public void requestLoop() {

	String numReq = ""; // id of request
	String idStr = "";
	int id = -1; // id sent by client, must match idClient

	try {

	    // waiting for request 0, that asks for client's id
	    numReq = br.readLine();
	    if ((numReq != null) && ("0".equals(numReq))) {
		// sending idClient to client
		ps.println(idClient);
	    }
	    else {
		System.out.println("Client "+idClient+" does not ask for its id correctly. Aborting");
		return;
	    }
	    
	    while(true) {
		numReq = br.readLine();
		if (numReq == null) { // violent close from client
		    break;
		}
		else if (numReq == "") { // explicit close
		    break;
		}

		//reading and testing id sent by client
		idStr = br.readLine();
		boolean err = false;
		try {
		    id = Integer.parseInt(idStr);
		}
		catch(NumberFormatException e) {
		    err = true;
		}

		if (err) {
		    ps.println("REQ_ERR");	
		}
		else if ((id != idClient) || (!data.exists(id))) {  
		    ps.println("NOID_ERR");
		}
		else if (numReq.equals("1")) {
		    requestStorePos(id);
		}
		else if (numReq.equals("2")) {
		    requestGetPathLength(id);
		}
		else if (numReq.equals("3")) {
		    requestFindPos(id);
		}
		else if (numReq.equals("4")) {
		    requestSlope(id);
		}		
		else if (numReq.equals("5")) {
		    requestPosOthers(id);
		}		
		
	    }
	    System.out.println("end of request loop");
	}
	catch(IOException e) {
	    System.out.println("problem with receiving request: "+e.getMessage());	    
	}
    }

    protected void requestStorePos(int id) throws IOException {
	System.out.println("processing request STORE POSITION");      

	boolean err = false;
	double x=0,y=0,z=0;	
	String params = br.readLine();
	String[] lst = params.split(",");
	if (lst.length != 3) {
	    err = true;
	}
	else {
	    try {
		x = Double.parseDouble(lst[0]);
		y = Double.parseDouble(lst[1]);
		z = Double.parseDouble(lst[2]);
	    }
	    catch(NumberFormatException e) {
		err = true;
	    }
	}
	if (err) {
	    ps.println("REQ_ERR");
	    return;
	}

	data.addPosition(id,x,y,z);
	ps.println("OK");
    }

    protected void requestGetPathLength(int id) throws IOException {
	System.out.println("processing request GET PATH LENGTH");
	
	double length = data.getPathLength(id);
	ps.println(length);
    }

    protected void requestFindPos(int id) throws IOException {
	System.out.println("processing request FIND POSITION");

	boolean err = false;
	double prec=0,x=0,y=0,z=0;	
	String params = br.readLine();
	String[] lst = params.split(",");
	
	if (lst.length != 4) {
	    err = true;
	}
	else {
	    try {
		prec = Double.parseDouble(lst[0]);		
		x = Double.parseDouble(lst[1]);
		y = Double.parseDouble(lst[2]);
		z = Double.parseDouble(lst[3]);
	    }
	    catch(NumberFormatException e) {
		err = true;
	    }
	}
	if (err) {
	    ps.println("REQ_ERR");
	    return;
	}

	boolean ok = data.findPosition(id,x,y,z,prec);

	if (ok) {
	    ps.println("TRUE");
	}
	else {
	    ps.println("FALSE");		
	}
    }

    protected void requestSlope(int id) throws IOException {
	System.out.println("processing request SLOPE");

	double[] slope = data.getSlope(id);
	ps.println(slope[0]);
	ps.println(slope[1]);	
    }
    
    protected void requestPosOthers(int id) throws IOException {
	System.out.println("processing request POSITION OF OTHERS");
	String posOthers = data.getPosOthers(id);
	ps.println(posOthers);
    }
    
}

		
